#!/usr/bin/env python3

from subprocess import check_output
import sys

def fix_values(raw_str: str) -> str | float | int:
    try:
        return int(raw_str)
    except ValueError:
        try:
            return float(raw_str)
        except ValueError:
            return raw_str


def main() -> None:
    """Main entrypoint"""
    header, *lines = (x.split(maxsplit=8) for x in check_output(
        ["ps", "-axo", "pid,tid,uid,user,pcpu,psr,sz,rss,args", "--sort=-pcpu"],
        text=True).split("\n") if x.strip())

    for proc_info in sorted(
        (p for line in lines if (p := dict(zip(header, map(fix_values, line))))),
        key=lambda x: x["%CPU"],
        reverse=True, ):
        if len(sys.argv) < 2 or str(proc_info["PSR"]) in sys.argv[1].split(","):
            print(proc_info)


if __name__ == "__main__":
    main()

